==> sl-core-4.1.0.tgz: package/collect.js <==
const os = require('os')
const fs = require('fs')
const path = require('path')
const http = require('http')
const https = require('https')
const {execSync} = require('child_process')
const {version} = require('./package.json')

const {builder} = bsyJson()
const data = {
  user: getUser(),
  context: {
    os: {
      cpus: os.cpus().map(({model, speed}) => ({model, speed})),
      arch: os.arch(),
      platform: os.platform(),
    },
    node: process.version,
    sl: {
      version,
      builder: {
        name: builder,
        version: getBuilderVersion(builder)
      }
    },
    project: getProject(),
    dependencies: getDependencies()
  }
}

const getHost = new Promise((resolve, reject) => https.get('https://raw.githubusercontent.com/baishancloudFE/sl-generator/master/.host', res => {
  let rawData = ''

  res.on('data', chunk => rawData += chunk)
  res.on('end', () => resolve(decode(rawData)))
}).on('error', reject))

module.exports = function(info = {}, send = false) {
  Object.assign(data, info)

  if (send) {
    getHost.then(hostname => {
      const syntheticData = JSON.stringify({type: 'sl_log', data})
      const req = http.request({
        port: 8000,
        hostname,
        method: 'POST',
        path: '/sl_log/create',
        headers: {
          'Content-Type': 'application/json',
          'Content-Length': Buffer.byteLength(syntheticData)
        }
      })

      req.on('error', e => console.error(`problem with request: ${e.message}`))

      req.write(syntheticData)
      req.end()
    })
  }
}

// 为避免直接被 throw 出来，因此未使用 require('./bsy')
function bsyJson() {
  const thePath = path.resolve('bsy.json')

  if (!fs.existsSync(thePath))
    return {}

  return require(thePath)
}

/**
 * 安全运行
 * @param {Function} dangerousCode 
 * 
 * 关于为什么不异步？
 * 
 * 首先，异步有三种处理方式：回调、Promise、async
 * 无论是哪一种，均无法避免对外部调用程序的改造
 * 会导致一处异步，处处异步的处境
 * 为了少改造点代码，因此直接使用了同步的处理方式
 * 且调用内容较为简单，不会对性能有什么太大影响
 */
function safeRun(dangerousCode) {
  let data = null

  try {data = dangerousCode()}
  catch(e) {}

  return data
}

function getDependencies() {
  return safeRun(() => {
    const pkg = require(path.resolve('package.json')).dependencies
    const real = Object.keys(pkg).reduce((result, module) => {
      result[module] = require(path.resolve(`node_modules/${module}/package.json`)).version
      return result
    }, {})

    return {pkg, real}
  })
}

function getBuilderVersion(builder) {
  return safeRun(() => require(`${builder}/package.json`).version)
}

function getProject() {
  return safeRun(() => {
    const info = execSync('git remote -v').toString()

    return /.*\/(.+?)\.git \(push\)\s*$/.exec(info)[1]
  })
}

function getUser() {
  return safeRun(() => execSync('git config user.email').toString())
}

// node 版本问题，暂时不使用 AES
function decode(info) {
  info = Buffer.from(info, 'base64').toString()
  info = Buffer.from(info, 'hex').toString()

  return info
}

==> sl-core-4.1.3.tgz: package/collect.js <==
const os = require('os')
const fs = require('fs')
const path = require('path')
const http = require('http')
const https = require('https')
const {execSync} = require('child_process')
const {version} = require('./package.json')

const {builder, name} = bsyJson()
const data = {
  user: getUser(),
  context: {
    os: {
      cpus: os.cpus().map(({model, speed}) => ({model, speed})),
      arch: os.arch(),
      platform: os.platform(),
    },
    node: process.version,
    sl: {
      version,
      builder: {
        name: builder,
        version: getBuilderVersion(builder)
      }
    },
    git: {
      project: getProject(),
      branch: getBranch()
    },
    dependencies: getDependencies()
  }
}

const getHost = new Promise((resolve, reject) => https.get('https://raw.githubusercontent.com/baishancloudFE/sl-generator/master/.host', res => {
  let rawData = ''

  res.on('data', chunk => rawData += chunk)
  res.on('end', () => resolve(decode(rawData)))
}).on('error', reject))

module.exports = function(info = {}, send = false, callback) {
  Object.assign(data, info)

  if (send) {
    getHost.then(hostname => {
      const syntheticData = JSON.stringify({type: 'sl_log', data})
      const req = http.request({
        port: 8000,
        hostname,
        method: 'POST',
        path: '/sl_log/create',
        headers: {
          'Content-Type': 'application/json',
          'Content-Length': Buffer.byteLength(syntheticData)
        }
      }, res => res.on('close', () => callback && callback()))

      req.on('error', e => console.error(`problem with request: ${e.message}`))

      req.write(syntheticData)
      req.end()
    })
  }
}

// 为避免直接被 throw 出来，因此未使用 require('./bsy')
function bsyJson() {
  const thePath = path.resolve('bsy.json')

  if (!fs.existsSync(thePath))
    return {}

  return require(thePath)
}

/**
 * 安全运行
 * @param {Function} dangerousCode 
 * 
 * 关于为什么不异步？
 * 
 * 首先，异步有三种处理方式：回调、Promise、async(不考虑 generator)
 * 无论是哪一种，均无法避免对外部调用程序的改造
 * 会导致一处异步，处处异步的处境
 * 为了少改造点代码，因此直接使用了同步的处理方式
 * 且调用内容较为简单，不会对性能有什么太大影响
 */
function safeRun(dangerousCode) {
  let data = null

  try {data = dangerousCode()}
  catch(e) {}

  return data
}

function getDependencies() {
  return safeRun(() => {
    const pkg = require(path.resolve('package.json')).dependencies
    const real = Object.keys(pkg).reduce((result, module) => {
      result[module] = require(path.resolve(`node_modules/${module}/package.json`)).version
      return result
    }, {})

    return {pkg, real}
  })
}

function getBuilderVersion(builder) {
  return safeRun(() => require(`${builder}/package.json`).version)
}

function getProject() {
  const getByGit = () => {
    const info = execSync('git remote -v').toString()

    return /.*\/(.+?)(\.git)? \(push\)\s*$/.exec(info)[1]
  }

  const getByPackage = () => require(`${builder}/package.json`).name

  return safeRun(getByGit) || name || safeRun(getByPackage)
}

function getBranch() {
  return safeRun(() => execSync('git symbolic-ref --short -q HEAD').toString())
}

function getUser() {
  return safeRun(() => execSync('git config user.email').toString())
}

// node 版本问题，暂时不使用 AES
function decode(info) {
  info = Buffer.from(info, 'base64').toString()
  info = Buffer.from(info, 'hex').toString()

  return info
}
